/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM and part of the
    HZDR Multiphase Addon for OpenFOAM.

    Copyright (C) 2022-2023 OpenFOAM Foundation
    and Helmholtz-Zentrum Dresden-Rossendorf

    OpenFOAM and the HZDR Multiphase Addon for OpenFOAM are free software:
    you can redistribute it and/or modify it under the terms of the
    GNU General Public License as published by the Free Software Foundation,
    either version 3 of the License, or (at your option) any later version.

    OpenFOAM and the HZDR Multiphase Addon for OpenFOAM are distributed in the
    hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM and the HZDR Multiphase Addon for OpenFOAM.
    If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::dropletEntrainment

Description
    Droplet entrainment model

    The area-specific mass transfer rate is computed based on a Weber number
    criterion. The model takes into account that the entrainment rate does not
    exceed a certain maximum. The entrainment vanishes on fine meshes assuming
    that the entrainment is resolved at a certain refinement level.

    The entrainment rate is computed by

    \f[
        \varepsilon
        = \frac{ \rho_f \nu_f }{ L_c } \min\left( C \,We, \; 0.5 \right)
    \f]

    with the capillary length

    \f[
        L_c = \sqrt{ \frac{\sigma}{g \, \rho_f} }
    \f]

    and the Weber number computed from the interfacial velocity jump:

    \f[
        We = \frac{\rho_g U_\text{rel}^2 L_c}{\sigma}, \quad
        U_\text{rel} = || U^+ - U^- ||
    \f]

    The constant \f$C\f$ is an empirical parameter.

    The corresponding volumetric mass source is obtained by:

    \f[
        ||\nabla\alpha||\varepsilon
    \f]

    The transfer is distributed over the interface, i.e. in cells with nonzero
    values of the interfacial area density \f$||\nabla\alpha||\f$ (IAD). Such a
    cell can only contribute to the transfer if it contains some pairContinuous.
    If not, the corresponding proportion of IAD is approximatively added to a
    neighbour cell still containing pairContinuous to realise the whole transfer
    rate.

    References:
    \verbatim
        Wang, L., Krull, B., Meller, R., Schlegel, F. (in preparation).
        Simulation of droplet entrainment in annular flow with a morphology
        adaptive two-fluid model
    \endverbatim

Usage
    \table
        Property    | Description               | Required | Default
        coefficient | dimensionless coefficient | no       | 1e-2
    \endtable

SourceFiles
    dropletEntrainment.C

\*---------------------------------------------------------------------------*/

#ifndef dropletEntrainment_H
#define dropletEntrainment_H

#include "entrainmentModel.H"
#include "segregatedPhaseInterface.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                     Class dropletEntrainment Declaration
\*---------------------------------------------------------------------------*/

class dropletEntrainment
:
    public entrainmentModel
{
    // Private Data

        //- Dimensionless coefficient
        const dimensionedScalar coefficient_;


protected:

    // Protected Member Functions

        //- Returns the droplet transfer rate
        virtual tmp<volScalarField> rate(const phaseModel&) const;


public:

    //- Runtime type information
    TypeName("dropletEntrainment");


    // Constructors

        //- Construct from components
        dropletEntrainment
        (
            const dictionary& dict,
            const phaseInterface& interface
        );


    //- Destructor
    virtual ~dropletEntrainment();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
