#!/bin/bash
#---------------------------------*- sh -*-------------------------------------
# ==  == ====== ====   ====    |
#                   \\     ||  | HZDR Multiphase Addon for OpenFOAM
# ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
# ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
# ==  == ====== ====   ==  ==  |
#------------------------------------------------------------------------------
# License
#     This file is part of the HZDR Multiphase Addon for OpenFOAM.
#
#     Copyright (C) 2018-2023 Helmholtz-Zentrum Dresden-Rossendorf
#
#     HZDR Multiphase Addon for OpenFOAM is free software:
#     you can redistribute it and/or modify it under the terms of the
#     GNU General Public License as published by the Free Software Foundation,
#     either version 3 of the License, or (at your option) any later version.
#
#     HZDR Multiphase Addon for OpenFOAM is distributed in the
#     hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#     implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the HZDR Multiphase Addon for OpenFOAM.
#     If not, see <http://www.gnu.org/licenses/>.
#
# Description
#     Script launches a OpenFOAM docker container from the docker registry.
#
#------------------------------------------------------------------------------

PROGNAME=$(basename "$0")

usage() {
   while [ "$#" -ge 1 ]; do echo "$1"; shift; done
   cat<<USAGE

usage: $PROGNAME [OPTIONS] IMAGE

options:
  -h, --help                        help
  -d  --desktop                     automatically choose display server
  -v, --volume SOURCE:TARGET        host directory to mount
      --ssh                         enable ssh connection for the container
      --no-pull                     disables automatic pull for the container
      --no-rm                       disables automatic remove of container on exit

* Launches the specified OpenFOAM Docker image. Run \`docker run --help\` for more options.

**** DISPLAY SERVER INSIDE DOCKER ****

On Linux operating systems X-Window-Servers and Wayland-Servers are supported.
On Mac OS install XQuartz according to the installation instructions. Depending
on the operating system and the docker installation the security settings for
X-Window-Server have to be modifed:

X-Window Server on Linux OS with none rootless docker installation:

  Allow access of the user to X-Window Server: 'xhost +SI:localuser:$(id -un)'

X-Window-Server (XQuartz) on Mac OS:

  Enable "Allow connections from network clients" in XQuartz settings and
  allow access to X-Window-Server: 'xhost + 127.0.0.1'

USAGE
   exit 1
}

#Check if we can use podman
if [ -x "$(command -v podman)" ]; then
    DOCKER_EXEC="podman"
elif [ -x "$(command -v docker)" ]; then
    DOCKER_EXEC="docker"
else
    usage "Found neither Podman nor Docker"
fi

#Check for at least one argument
[ "$#" -eq 0 ] && usage "No docker image specified."

# Default options for docker run command
DOCKER_OPTIONS=(
    "--interactive"
    "--tty"
    "--rm"
    "--pull=always"
    "--env LOCAL_USER_NAME=$USER"
    "--volume config:/root/.config"
    "--volume /etc/localtime:/etc/localtime:ro"
    )

# Options for docker run command from command line
CMD_OPTIONS=()

# Parse options
while [ "$#" -gt 0 ]
do
    case "$1" in
    -v | --volume)
        [ "$#" -ge 2 ] || usage "'$1' option requires an argument"
        SOURCE=${2%:*}
        TARGET=${2##*:}
        if [ -d "$SOURCE" ]; then
            DOCKER_OPTIONS+=("--volume $SOURCE:$TARGET")
        else
            usage "No directory exists: $SOURCE"
        fi
        shift 2
        ;;
    --no-pull)
        # Position of --pull='always' in array
        DOCKER_OPTIONS[3]="--pull=never"
        shift
        ;;
    --no-rm)
        # Position of --rm in array
        unset 'DOCKER_OPTIONS[2]'
        shift
        ;;
    --gpus)
        # Position of --device=/dev/dri:/dev/dri in array
        unset 'DOCKER_OPTIONS[4]'
        CMD_OPTIONS+=("$1" "$2")
        shift 2
        ;;
    --ssh)
        #Check for ssh agent configuration
        if [ -z "$SSH_AUTH_SOCK" ]; then
            eval "$(ssh-agent -s)"
        fi
        DOCKER_OPTIONS+=("--env SSH_AUTH_SOCK=$SSH_AUTH_SOCK")
        DOCKER_OPTIONS+=("--volume $(dirname "$SSH_AUTH_SOCK"):$(dirname "$SSH_AUTH_SOCK")")
        shift
        ;;
    -d | --desktop)
        DOCKER_OPTIONS+=("--volume /tmp/.X11-unix:/tmp/.X11-unix")

        if [ "$(uname)" = "Darwin" ]
        then
            DOCKER_OPTIONS+=("--env DISPLAY=host.docker.internal:0")
        else
            if [ -n "$XDG_SESSION_TYPE" ]
            then
                DOCKER_OPTIONS+=("--device=/dev/dri:/dev/dri")

                if [ "$XDG_SESSION_TYPE" = "x11" ]
                then
                    DOCKER_OPTIONS+=("--env DISPLAY=$DISPLAY")
                elif [ "$XDG_SESSION_TYPE" = "wayland" ]
                then
                    if [ -n "$DISPLAY" ]
                    then
                        DOCKER_OPTIONS+=("--env DISPLAY=$DISPLAY")
                    else
                        usage "XWayland is not running. Please install it."
                    fi
                else
                    usage "Unkown display manager: $XDG_SESSION_TYPE"
                fi
            else
                usage "Cannot determine display manager"
            fi
        fi
        shift
        ;;
    -h | --help)
        usage
        ;;
    *)
        CMD_OPTIONS+=("$1")
        shift
        ;;
    esac
done

# Extract docker image
if [ ${#CMD_OPTIONS[@]} -eq 1 ]
then
    DOCKER_IMAGE=${CMD_OPTIONS[0]}
    CMD_OPTIONS=()
else
    DOCKER_IMAGE=${CMD_OPTIONS[-1]}
    unset 'CMD_OPTIONS[-1]'
fi

# Create docker container
# shellcheck disable=SC2086,SC2048
CONTAINERID=$($DOCKER_EXEC create ${DOCKER_OPTIONS[*]} ${CMD_OPTIONS[*]} "$DOCKER_IMAGE")

# Get git configuration file
if [ -n "$CONTAINERID" ]
then
    if [ -f "$HOME/.gitconfig" ]
    then
        $DOCKER_EXEC cp "$HOME/.gitconfig" "$CONTAINERID:/root/.gitconfig"
    else
        echo "Warning: Git config not found in $HOME/.gitconfig. Is git installed?"
    fi

    $DOCKER_EXEC start -i "$CONTAINERID"
else
    usage "Unable to launch $DOCKER_IMAGE."
fi
